'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var VueFormGeneratorGraphQL = function () {
  function VueFormGeneratorGraphQL() {
    _classCallCheck(this, VueFormGeneratorGraphQL);
  }

  _createClass(VueFormGeneratorGraphQL, [{
    key: 'buildModel',

    /**
     * Build flat model based on schema with groups
     * @param schema
     * @param model - initial model
     * @returns {{}}
     */
    value: function buildModel(schema) {
      var model = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      if (!schema) {
        throw new Error('[vue-form-generator-graphql] $buildModel: Schema is not presented');
      } else if ((typeof schema === 'undefined' ? 'undefined' : _typeof(schema)) !== 'object') {
        throw new TypeError('[vue-form-generator-graphql] $buildModel: Type of schema expected "object" instead of ' + (typeof schema === 'undefined' ? 'undefined' : _typeof(schema)));
      } else if (!!schema.groups && schema.groups.length === 0 && !schema.fields) {
        throw new Error('[vue-form-generator-graphql] $buildModel: Schema is empty');
      }

      // If was given array-like object
      if (schema.groups && !schema.groups.length) {
        schema.groups = Object.keys(schema.groups).map(function (item) {
          return schema.groups[item];
        });
      }

      // Factory ?
      if (schema.groups && schema.groups.length) {
        schema.groups.forEach(function (group) {
          model = Util.createModel(group, model);
        });
      } else if (schema.fields) {
        model = Util.createModel(schema, model);
      }

      return model;
    }

    /**
     * Method query builds query/mutation string for usage in GraphQL
     * Method vars builds object with variables
     * @param schema
     * @param model
     * @param options.name - required - name of query/mutation in UpperCamelCase
     * @param options.type (mutation) - 'query' or 'mutation'
     * @param options.fields (status, message) - fields to fetch as response
     * @param options.label (id) - label of ID in model objects
     * @param options.customTypesMap - object of custom types
     * @returns {*}
     */

  }, {
    key: 'buildQuery',
    value: function buildQuery(schema, model, options) {
      var opts = {};

      if (!schema) {
        throw new Error('[vue-form-generator-graphql] $buildQuery: Schema must be defined');
      }

      if (!options.name) {
        throw new Error('[vue-form-generator-graphql] $buildQuery: Query/mutation name must be defined');
      }

      if (!options.type) {
        opts.type = 'mutation';
      }

      if (!options.fields) {
        opts.fields = 'status, message';
      }

      if (!options.label) {
        opts.label = 'id';
      }

      Object.assign(opts, options);

      var util = new Util(schema, model, opts);

      return {
        query: function query() {
          return opts.type + ' Fetch' + opts.name + ' (\n          ' + util.defineVariables() + '\n        ) {\n          ' + util.convertName2type() + ' (\n            ' + util.bindVariables() + '\n          ) {\n            ' + opts.fields + '\n          }\n        }';
        },
        vars: function vars(args) {
          return util.makeVars(args);
        },
        getType: function getType() {
          return util.convertName2type();
        }
      };
    }
  }]);

  return VueFormGeneratorGraphQL;
}();

var Util = function () {
  function Util(schema, model, opts) {
    _classCallCheck(this, Util);

    this.schema = schema;
    this.model = model;
    this.opts = opts;

    this.defineVariableType = Util.defineVariableType.bind(this);
  }

  /**
   * Checks variable type considering custom types and returns this type
   * @param item
   * @returns {*}
   */


  _createClass(Util, [{
    key: 'convertName2type',


    /**
     * Converts query Name to Type (UpperCamelCase to lower_case)
     * @returns {string}
     */
    value: function convertName2type() {
      return this.opts.name.replace(/([A-Z])/g, '_$1').replace('_', '').toLowerCase();
    }
  }, {
    key: 'defineVariables',


    /**
     * Creates string of variables with its types
     * @returns {string}
     */
    value: function defineVariables() {
      var _this = this;

      return Object.keys(this.model).map(function (item) {
        return '$' + item + ': ' + _this.defineVariableType(item);
      }).join(',\r\n');
    }
  }, {
    key: 'bindVariables',


    /**
     * Bind variables with its types
     * @returns {string}
     */

    // $order: order_input
    // order_input: $order
    value: function bindVariables() {
      var _this2 = this;

      return Object.keys(this.model).map(function (item) {
        if (_this2.opts.customTypesMap && _this2.opts.customTypesMap[item]) {
          return _this2.defineVariableType(item) + ': $' + item;
        }
        return item + ': $' + item;
      }).join(',\r\n');
    }
  }, {
    key: 'makeVars',


    /**
     * Cleans model
     * @returns {{}}
     */
    value: function makeVars() {
      var _this3 = this;

      var includeNulls = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

      var result = {};

      Object.keys(this.model).map(function (item) {
        if (_typeof(_this3.model[item]) === 'object' && _this3.model[item] !== null) {
          // Object
          if (Object.keys(_this3.model[item]).some(function (el) {
            return _this3.model[item][el];
          }) || includeNulls) result[item] = _this3.model[item];
        } else if (_this3.model[item] === null && includeNulls) {
          // Null
          result[item] = null;
        } else if (_this3.model[item] !== null) {
          // Has value
          result[item] = _this3.model[item];
        }
      });

      return result;
    }
  }], [{
    key: 'defineVariableType',
    value: function defineVariableType(item) {
      var preDefinedType = findValueType.call(this, item);

      function findValueType(item) {
        var result = void 0;

        for (var i in this.schema.groups) {
          var group = this.schema.groups[i];

          result = group.fields.find(function (field) {
            return field.model === item;
          });

          if (!!result) {
            return result.valueType;
          }
        }
      }

      if (preDefinedType) {
        return preDefinedType;
      } else if (this.opts.customTypesMap && this.opts.customTypesMap[item]) {
        return this.opts.customTypesMap[item];
      } else if (typeof this.model[item] === 'string') {
        return 'String';
      } else if (typeof this.model[item] === 'number' || this.model[item] === null || _typeof(this.model[item]) === 'object' && typeof this.model[item][this.opts.label] === 'number') {
        return this.model[item] % 1 === 0 ? 'Int' : 'Float';
      } else if (typeof this.model[item] === 'boolean') {
        return 'Boolean';
      } else if (this.model[item] && this.model[item].length) {
        // Array
        switch (_typeof(this.model[item][0])) {
          case 'string':
            return '[String]';
            break;
          case 'number':
            return '[Int]';
            break;
          default:
            console.warn('[vue-form-generator-graphql] Type of elements in array ' + item + ' is not defined');
        }
      } else {
        return item;
      }
    }
  }, {
    key: 'createModel',
    value: function createModel(group, model) {
      if (!group || !group.fields) {
        throw new Error('[vue-form-generator-graphql] createModel: No fields list provided');
      }

      group.fields.forEach(function (field) {
        if (!!field.model && !!field.default) {
          model[field.model] = field.default;
        } else if (!!field.model && !!field.selectOptions) {
          field.values.forEach(function (value) {
            if (value.id === field.selectOptions.default) {
              model[field.model] = {
                id: value.id,
                name: value.name
              };
            }
          });
        } else {
          model[field.model] = null;
        }
      });

      return model;
    }
  }]);

  return Util;
}();

var vfgg = new VueFormGeneratorGraphQL();

exports.default = VueFormGeneratorGraphQL.install = function (Vue) {
  Vue.prototype.$buildModel = function (schema) {
    return vfgg.buildModel(schema);
  };
  Vue.prototype.$buildQuery = function (schema, model, options) {
    return vfgg.buildQuery(schema, model, options);
  };
};